##poisson regression

# packages
library(readxl)
library(sandwich) # for robust standard errors
library(msm) # for exponentiates using the delta method

# function for extracting and transforming the parameters of interest from a poisson fit
# loosely based off code in https://stats.idre.ucla.edu/r/dae/poisson-regression/
# n_covariates are the number of covariates in the model
# the function is NOT general, it only works for 1 covariate (year) or 6 (the 5 group estaimates + 1 for year)
parameters_poiss = function(fit, n_covariates){
  
  # calculating robust standard errors
  cov_fit_level = vcovHC(fit, type="HC0")
  coef_fit_level = coef(fit)
  std_err = sqrt(diag(cov_fit_level))
  r_est = cbind(Estimate = coef_fit_level, "Robust SE" = std_err,
                "Pr(>|z|)" = 2 * pnorm(abs(coef_fit_level/std_err), lower.tail=FALSE),
                LL = coef_fit_level - 1.96 * std_err,
                UL = coef_fit_level + 1.96 * std_err)
  
  # exponentiating to get relative risks / incidence rates
  # because the code is specific to the length of these specific models
  # the code is NOT general for use for other poisson models
  if(n_covariates == 1){
    se = deltamethod(list(~exp(x1), ~exp(x2)), 
                     coef_fit_level, cov_fit_level)
  } else if(n_covariates == 7){
    se = deltamethod(list(~exp(x1), ~exp(x2), ~exp(x3), ~exp(x4), ~exp(x5), ~exp(x6), ~exp(x7), ~exp(x8)), 
                     coef_fit_level, cov_fit_level)    
  } else if(n_covariates == 2)
    se = deltamethod(list(~exp(x1), ~exp(x2), ~exp(x3)), 
                     coef_fit_level, cov_fit_level)  
  
  ## exponentiate old estimates dropping the p values
  p_values = r_est[, 3]
  rexp_est = exp(r_est[, -3])
  ## replace SEs with estimates for exponentiated coefficients
  rexp_est[, "Robust SE"] = se
  # add p-values
  rexp_est = cbind(rexp_est, p_values)
  rexp_est
}

# read data
export_months = read_excel("poisson_months.xlsx")
# change numeric variables to factor
# so that they are treated as categorical in the model
export_months$year = as.factor(export_months$year)
# injuries
poisson.months.yearonly = glm (formula = n_all_injuries ~ year, export_months, family = "poisson")
poisson.months = glm (formula = n_all_injuries ~ year + n_matches, export_months, family = "poisson")
summary(poisson.months.yearonly)    
summary(poisson.months)

# fetch parameters with our function
param.monthly = parameters_poiss(poisson.months.yearonly, 1)
param.monthly.matchadjusted = parameters_poiss(poisson.months, 2)

# repeat for match injuries
poisson.months.yearonly.matchinj = glm (formula = n_matchinjuries ~ year, export_months, family = "poisson")
poisson.months.matchinj = glm (formula = n_matchinjuries ~ year + n_matches, export_months, family = "poisson")

summary(poisson.months.yearonly.matchinj)    
summary(poisson.months.matchinj)

param.monthly.matchinj = parameters_poiss(poisson.months.yearonly.matchinj, 1)
param.monthly.matchadjusted.matchinj = parameters_poiss(poisson.months.matchinj, 2)

# repeat for year and group
export_R2 = read_excel("export_R2.xlsx")
export_R2$team = as.factor(export_R2$team)
export_R2$year = as.factor(export_R2$year)


# injuries
poisson.inj <- glm(Injuries ~ year, export_R2, family = "poisson")
summary(poisson.inj)    
poisson.inj.group <- glm (formula = Injuries ~ year + team, export_R2, family = poisson (link ="log"))
summary(poisson.inj.group)    

poisson.inc <- glm (formula = Incidence ~ year, export_R2, family = poisson (link ="log"))
summary(poisson.inc)  
poisson.inc.group <- glm (formula = Incidence ~ year + team, export_R2, family = poisson (link ="log"))
summary(poisson.inc.group)    

# use function to get parameters
param.inj = parameters_poiss(poisson.inj, n_covariates = 1)
param.inj.group = parameters_poiss(poisson.inj.group, n_covariates = 7)
param.inc = parameters_poiss(poisson.inc, n_covariates = 1)
param.inc.group = parameters_poiss(poisson.inc.group, n_covariates = 7)

# function to save results of fits
save_res = function(fit_summary, name){
  out_fit = capture.output(fit_summary)
  cat(name, out_fit, file="summary_of_fits.txt", sep="\n", append=TRUE)
}

save_res(param.inj, "Injuries 2019 vs. 2020")
save_res(param.inj.group, "Injuries Team Differences")
save_res(param.inc, "Incidence 2019 vs. 2020")
save_res(param.inc.group, "Incidence Team Differences")
save_res(param.monthly, "Monthly Injury")
save_res(param.monthly.matchadjusted, "Monthly Injury Adjusted for N Matches")
save_res(param.monthly.matchinj, "Monthly Match Injuries")
save_res(param.monthly.matchadjusted.matchinj, "Monthly Match Injuries Adjusted for N Matches")